import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:xcommonwidgets/xcommonwidgets.dart';
import 'package:xfxbase/XFDataItem.dart';
import 'package:xfxwidgets/xPDFviewer.dart';
import 'package:flutter_cached_pdfview/flutter_cached_pdfview.dart';
import 'package:flutter_email_sender/flutter_email_sender.dart';
import 'package:xtoolspackage/xtoolspackage.dart';

abstract class XfxUtils {
  static String GetStringFromValue(dynamic value, {XProp? colForXDataItem = null}) {
    if (value is XFDataItem) {
      var str = value[colForXDataItem!.col_Name].toString();
      return str;
    } else if (value is XEnumDef) {
      return value.label;
    } else
      return value.toString();
  }

  static String xddValueToString(Object? value, Iterable<XProp> colsOfXFdataItemToShow) {
    if (value is XFDataItem) {
      return xddValueToString_XFD(value, colsOfXFdataItemToShow);
    } else if (value is XEnumDef) {
      return value.label;
    } else
      return value.toString();
  }

  static String xddValueToString_XFD(XFDataItem value, Iterable<XProp> colsOfXFdataItemToShow) {
    var str = "";
    colsOfXFdataItemToShow.forEach((element) {
      str += (str.isEmpty ? "" : " ") + value[element.col_Name].toString();
    });
    return str;
  }

  Future xGenPDF_UI(void Function(void Function()) parentState, BuildContext context, Future<List<String?>?> Function() getUrls_API, Future<Uint8List> Function() getBytesDoc_API, XFDataItem obj_Parent, {String? waiterMessage, bool? isBusy_ParentPage, List<String?>? emailsCli, String? email_Body, String? email_Subject, List<String>? liAddons_FilesPath}) async {
    parentState(() {
      waiterMessage != null ? waiterMessage = "Sto Preparando il Preventivo da Visualizzare" : null;
      isBusy_ParentPage != null ? isBusy_ParentPage = true : null;
    });
    var urls = await getUrls_API();
    if (urls == null || urls.isEmpty) {
      return XAlertDialog.alertClose(context, titleStyle: XStyles.xStyTextForDescr(textColor: Colors.amber), titleText: "Scusa ma non riesco a generare il Preventivo, controlla i dati o se non lo hai ancora fatto inserisci un dettaglio.");
    } else {
      await xOpenPDF(urls, context, getBytesDoc_API, obj_Parent, emailsCli: emailsCli, email_Body: email_Body, email_Subject: email_Subject, liAddons_FilesPath: liAddons_FilesPath);
    }
    parentState(() {
      waiterMessage != null ? waiterMessage = "" : null;
      isBusy_ParentPage != null ? isBusy_ParentPage = false : null;
    });
  }

  Future xOpenPDF(List<String?>? urls, BuildContext context, Future<Uint8List> Function() getBytesDoc_API, XFDataItem obj_Parent, {List<String?>? emailsCli, String? email_Body, String? email_Subject, List<String>? liAddons_FilesPath}) async {
    List<XpdfScreenItem> liWidgetsPDF = List<XpdfScreenItem>.empty(growable: true);
    if (urls!.isEmpty) return;
    for (var x in urls) {
      var r = PDF(
        enableSwipe: true,
      ).fromUrl(x!);
      liWidgetsPDF.add(XpdfScreenItem(title: "Preventivo", widgetToView: r));
    }

    var bytesDoc = await getBytesDoc_API();
    await Navigator.push<void>(
        context,
        MaterialPageRoute<void>(
            builder: (BuildContext context) => XPDFScreen<XFDataItem>(
                liWidgetPDF: liWidgetsPDF,
                getDoc_Bytes: bytesDoc,
                itemInEdit: obj_Parent,
                sendEmailBTN_visible: emailsCli != null ? true : false,
                email: emailsCli != null
                    ? (XFDataItem item, liFiles) {
                        var liFilesUI = liFiles.map((e) => e.path).toList();
                        liAddons_FilesPath != null ? liFilesUI.addAll(liAddons_FilesPath) : null;
                        return Email(
                          body: email_Body ?? 'Buongiorno',
                          subject: email_Subject ?? 'Preventivo rif.',
                          recipients: emailsCli.map((e) => e ?? "").toList(),
                          attachmentPaths: liFilesUI,
                          isHTML: false,
                        );
                      }
                    : null)));
  }
}
